package java.util;

import java.io.ObjectInputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.security.Permission;
import java.io.ObjectStreamField;
import java.io.Serializable;
import java.security.PermissionCollection;

final class PropertyPermissionCollection extends PermissionCollection implements Serializable
{
    private transient Map<String, PropertyPermission> perms;
    private boolean all_allowed;
    private static final long serialVersionUID = 7015263904581634791L;
    private static final ObjectStreamField[] serialPersistentFields;
    
    public PropertyPermissionCollection() {
        this.perms = new HashMap(32);
        this.all_allowed = false;
    }
    
    @Override
    public void add(Permission obj) {
        if (!(obj instanceof PropertyPermission)) {
            throw new IllegalArgumentException("invalid permission: " + obj);
        }
        if (this.isReadOnly()) {
            throw new SecurityException("attempt to add a Permission to a readonly PermissionCollection");
        }
        PropertyPermission propertyPermission = (PropertyPermission)obj;
        String name = propertyPermission.getName();
        synchronized (this) {
            PropertyPermission propertyPermission2 = (PropertyPermission)this.perms.get(name);
            if (propertyPermission2 != null) {
                int mask = propertyPermission2.getMask();
                int mask2 = propertyPermission.getMask();
                if (mask != mask2) {
                    this.perms.put(name, new PropertyPermission(name, PropertyPermission.getActions(mask | mask2)));
                }
            }
            else {
                this.perms.put(name, propertyPermission);
            }
        }
        if (!this.all_allowed && name.equals("*")) {
            this.all_allowed = true;
        }
        return;
    }
    
    @Override
    public boolean implies(Permission permission) {
        if (!(permission instanceof PropertyPermission)) {
            return false;
        }
        PropertyPermission propertyPermission = (PropertyPermission)permission;
        int mask = propertyPermission.getMask();
        int n = 0;
        if (this.all_allowed) {
            PropertyPermission propertyPermission2;
            synchronized (this) {
                propertyPermission2 = (PropertyPermission)this.perms.get("*");
            }
            if (propertyPermission2 != null) {
                n |= propertyPermission2.getMask();
                if ((n & mask) == mask) {
                    return true;
                }
            }
        }
        String s = propertyPermission.getName();
        PropertyPermission propertyPermission2;
        synchronized (this) {
            propertyPermission2 = (PropertyPermission)this.perms.get(s);
        }
        if (propertyPermission2 != null) {
            n |= propertyPermission2.getMask();
            if ((n & mask) == mask) {
                return true;
            }
        }
        int lastIndex;
        for (int fromIndex = s.length() - 1; (lastIndex = s.lastIndexOf(".", fromIndex)) != -1; fromIndex = lastIndex - 1) {
            s = s.substring(0, lastIndex + 1) + "*";
            synchronized (this) {
                propertyPermission2 = (PropertyPermission)this.perms.get(s);
            }
            if (propertyPermission2 != null) {
                n |= propertyPermission2.getMask();
                if ((n & mask) == mask) {
                    return true;
                }
            }
        }
        return false;
    }
    
    @Override
    public Enumeration<Permission> elements() {
        synchronized (this) {
            return Collections.enumeration(this.perms.values());
        }
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        Hashtable<Object, Object> hashtable = new Hashtable(this.perms.size() * 2);
        synchronized (this) {
            hashtable.putAll(this.perms);
        }
        ObjectOutputStream.PutField putFields = objectOutputStream.putFields();
        putFields.put("all_allowed", this.all_allowed);
        putFields.put("permissions", hashtable);
        objectOutputStream.writeFields();
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws IOException, ClassNotFoundException {
        ObjectInputStream.GetField fields = objectInputStream.readFields();
        this.all_allowed = fields.get("all_allowed", false);
        Hashtable<Object, Object> hashtable = (Hashtable)fields.get("permissions", null);
        (this.perms = new HashMap(hashtable.size() * 2)).putAll(hashtable);
    }
    
    static {
        serialPersistentFields = new ObjectStreamField[] { new ObjectStreamField("permissions", Hashtable.class), new ObjectStreamField("all_allowed", Boolean.TYPE) };
    }
}
